"""
Algorithms related to generating and processing mazes, represented as boolean
numpy arrays.
"""
from __future__ import print_function, division
import matplotlib.pyplot as plt
import numpy as np


def wrap_inc(M, i):
    return i + 1 if i < M - 1 else 0


def wrap_dec(M, i):
    return i - 1 if i > 0 else M - 1


def shrink(w_old, n):
    if n < 1:
        raise Exception('Shrink factor >= 1')
    elif n == 1:
        return w_old
    elif n % 2 != 0:
        raise Exception('Shrink factor must be odd')
    M = w_old.shape[0]
    w_new = np.zeros(w_old.ndim * [M * n], dtype=w_old.dtype)
    mid = n // 2
    for x in range(M):
        x_ = x * n
        for y in range(M):
            y_ = y * n
            if w_old[x, y]:
                w_new[x_ + mid, y_ + mid] = True
                if w_old[wrap_inc(M, x), y]:
                    w_new[x_ + mid:x_ + n, y_ + mid] = True
                if w_old[wrap_dec(M, x), y]:
                    w_new[x_:x_ + mid, y_ + mid] = True
                if w_old[x, wrap_inc(M, y)]:
                    w_new[x_ + mid, y_ + mid:y_ + n] = True
                if w_old[x, wrap_dec(M, y)]:
                    w_new[x_ + mid, y * n:y_ + mid] = True
    return w_new


def make_offsets(dim):
    offsets = np.zeros([2 * dim, dim])
    offsets[:dim] = np.identity(dim)
    offsets[dim:] = -np.identity(dim)
    return offsets


def step(p, o, m, n=1):
    p_new = p + n * o
    p_new[p_new < 0] += m
    p_new[p_new > m - 1] -= m
    return p_new


def make_maze_dfs(M=27, dim=2, rng=None):
    """Generate a maze using the Depth-first search algorithm.

    http://en.wikipedia.org/wiki/Depth-first_search

    Parameters
    ----------
    M: integer.
        Size of the maze.
        The algorithm requires that the maze size is even.
    dim: integer.
        The spatial dimension.
    seed: integer or None.
        Seed for the random number generator.
        None will use a different seed for each call.

    Returns
    -------
    maze: boolean array, shape (M, M)
        Array defining the maze. Walls are represented by `True`.
    """
    if M % 2 != 0:
        raise Exception('Require Maze size to be even.')
    if rng is None:
        rng = np.random
    maze = np.zeros(dim * (M,), dtype=np.bool)
    pos = rng.randint(0, M, dim)
    maze[tuple(pos)] = True
    path = [pos]
    offsets = make_offsets(dim)
    while path:
        neighbs = []
        for offset in offsets:
            if not maze[tuple(step(pos, offset, M, 2))]:
                neighbs.append(offset)
        if neighbs:
            offset = neighbs[rng.randint(len(neighbs))]
            for i in range(2):
                pos = step(pos, offset, M)
                maze[tuple(pos)] = True
            path.append(pos)
        else:
            pos = path.pop()
    return maze


def main():
    maze = make_maze_dfs(28)
    plt.imshow(maze, interpolation='nearest')
    plt.show()

if __name__ == '__main__':
    main()



from __future__ import print_function, division
import sys
import matplotlib.pyplot as plt
from bannock.plot import plot


def plot_chi_fs():
    ax = plt.gca()
    plot.plot_chi_fs(sys.argv[1:], ax)
    plt.show()


def plot_chi_ks():
    ax = plt.gca()
    plot.plot_chi_ks(sys.argv[1:], ax)
    plt.show()


def plot_t_fracs():
    ax = plt.gca()
    plot.plot_t_fracs(sys.argv[1], ax)
    plt.show()


def plot_t_ks():
    ax = plt.gca()
    plot.plot_t_ks(sys.argv[1], ax)
    plt.show()


def plot_vis():
    plot.plot_vis(sys.argv[1])



from __future__ import print_function, division
import numpy as np
from clustrous import cluster
from agaro.measure_utils import (get_average_measure, group_by_key, t_measures,
                                 params, measures)


r_cluster_1d = 5.0
r_cluster_2d = 20.0


def _get_r_cluster(dim):
    """Find the cluster length scale appropriate for a model's dimension.

    Parameters
    ----------
    dim: int
        Dimension.

    Returns
    -------
    r_cluster: float
        Cluster length scale.
    """
    if dim == 1:
        return r_cluster_1d
    elif dim == 2:
        return r_cluster_2d


# Parameter getters


def get_chi(m):
    return m.chi


def get_time(m):
    return m.t


# Measure getters


def get_k(m):
    """Calculate the particle clumpiness for a model.

    Parameters
    ----------
    m: Model
        Model instance.

    Returns
    -------
    k: float
        Clumpiness measure.
    """
    labels = cluster.cluster_periodic(m.r, _get_r_cluster(m.dim), m.L)
    clust_sizes = cluster.cluster_sizes(labels)
    return cluster.clumpiness(clust_sizes), 0.0


def get_fracs(m):
    """Calculate an order parameter, f, the 'confinedness',
    representing the fraction of particles in each trap for a model.

    f = 1 means all particles are in a trap.
    f = 0 means the fraction is that expected for a uniform distribution.
    f < 0 means there are fewer particles than expected.

    Parameters
    ----------
    m: Model
        Model instance.

    Returns
    -------
    fs: list[float]
        Confinedness for each trap.
    """
    fracs_0 = m.walls.get_trap_areas() / m.walls.get_free_area()
    confinednesses = (m.walls.get_fracs(m.r) - fracs_0) / (1.0 - fracs_0)
    confinedness_errs = np.zeros_like(confinednesses)
    return confinednesses, confinedness_errs


# Measures over time


def t_ks(dirname):
    """Calculate the particle clumpiness over time
    for a model output directory.

    Parameters
    ----------
    dirname: str
        A model output directory path

    Returns
    -------
    ts: numpy.ndarray[dtype=float]
        Times.
    ks: numpy.ndarray[dtype=float]
        Particle clumpinesses.
    """
    ts, ks, _ = t_measures(dirname, get_time, get_k)
    return ts, ks


def t_fracs(dirname):
    """Calculate the trap confinedness over time
    for a model output directory.

    Parameters
    ----------
    dirname: str
        A model output directory path

    Returns
    -------
    ts: numpy.ndarray[dtype=float]
        Times.
    fs: numpy.ndarray[dtype=float, shape=(ts.shape[0], num_traps)]
        Trap confinednesses, where `num_traps` is the number of traps.
    """
    ts, fracs, _ = t_measures(dirname, get_time, get_fracs)
    return ts, fracs


# Parameter-measure relations


def chi_ks(dirnames, t_steady=None):
    """Calculate the particle clumpiness of a set of
    model output directories, and their associated chis.

    Parameters
    ----------
    dirnames: list[str]
        Model output directory paths.
    t_steady: None or float
        Time to consider the model to be at steady-state.
        The measure will be averaged over all later times.
        `None` means just consider the latest time.

    Returns
    -------
    chis: numpy.ndarray[dtype=float]
        Chemotactic sensitivities
    ks: numpy.ndarray[dtype=float]
        Particle clumpinesses.
    """
    chis = params(dirnames, get_chi)
    ks, _ = measures(dirnames, get_k, t_steady)
    return chis, ks


def chi_fs(dirnames, t_steady=None):
    """Calculate particle confinedness of a set of
    model output directories, and their associated chis.

    Parameters
    ----------
    dirnames: list[str]
        Model output directory paths.
    t_steady: None or float
        Time to consider the model to be at steady-state.
        The measure will be averaged over all later times.
        `None` means just consider the latest time.

    Returns
    -------
    chis: numpy.ndarray[dtype=float]
        Chemotactic sensitivities
    fs: numpy.ndarray[dtype=float]
        Particle confinednesses.
    """
    chis = params(dirnames, get_chi)
    fracs, _ = measures(dirnames, get_fracs, t_steady)
    return chis, fracs


def chi_ks_run_average(dirnames, t_steady=None):
    """Calculate the particle clumpiness of a set of
    model output directories, and their associated chis.
    Take the average over all directories with equal chi.

    Parameters
    ----------
    dirnames: list[str]
        Model output directory paths.
    t_steady: None or float
        Time to consider the model to be at steady-state.
        The measure will be averaged over all later times.
        `None` means just consider the latest time.

    Returns
    -------
    chis: numpy.ndarray[dtype=float]
        Chemotactic sensitivities
    ks: numpy.ndarray[dtype=float]
        Particle clumpinesses.
    """
    chi_groups = group_by_key(dirnames, 'chi')
    chis, ks = [], []
    for chi, dirnames in chi_groups.items():
        chis.append(chi)
        ks.append(np.mean([get_average_measure(dirname, get_k, t_steady)
                           for dirname in dirnames]))
    return np.array(chis), np.array(ks)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import setuptools
from setuptools import setup
from distutils.extension import Extension
from Cython.Build import cythonize
import numpy

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    'Cython',
    'numpy',
    'matplotlib',
    'ciabatta',
    'agaro',
    'spatious',
    'cellulist',
    'fealty',
    'clustrous',
]

test_requirements = [
    'Cython',
    'numpy',
    'ciabatta',
    'agaro'
    'spatious',
    'cellulist',
    'fealty',
    'clustrous',
]

extensions = cythonize([
    Extension("bannock.numerics", ["bannock/numerics.pyx"],
              include_dirs=[numpy.get_include()]),
])

console_scripts = [
    'bplot_chi_fs = bannock.utils.scripts:plot_chi_fs',
    'bplot_chi_ks = bannock.utils.scripts:plot_chi_ks',
    'bplot_t_fracs = bannock.utils.scripts:plot_t_fracs',
    'bplot_t_ks = bannock.utils.scripts:plot_t_ks',
    'bplot_t_pmeans = bannock.utils.scripts:plot_t_pmeans',
    'bplot_vis = bannock.utils.scripts:plot_vis',
]

setup(
    name='bannock',
    version='0.4.0',
    description="Agent-based simulations in confined environments",
    long_description=readme + '\n\n' + history,
    author="Elliot Marsden",
    author_email='elliot.marsden@gmail.com',
    url='https://github.com/eddiejessup/bannock',
    packages=setuptools.find_packages(exclude=['docs', 'tests']),
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='bannock',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    ext_modules=extensions,
    entry_points={
        'console_scripts': console_scripts,
    }
)
